#include "mbed.h"
#include "Servo.h"
#include "DevSPI.h"
#include "PowerStep01.h"

Servo flipping_arm_lower(PA_5);
Servo flipping_arm_upper(PA_6);
//Servo clamping_arm(PA_7);

Serial pc(USBTX, USBRX);

powerstep01_init_u_t init =
{
  /* common parameters */
  .cm.cp.cmVmSelection = POWERSTEP01_CM_VM_CURRENT, // enum powerstep01_CmVm_t
  582, // Acceleration rate in step/s2, range 14.55 to 59590 steps/s^2
  582, // Deceleration rate in step/s2, range 14.55 to 59590 steps/s^2
  488, // Maximum speed in step/s, range 15.25 to 15610 steps/s
  0, // Minimum speed in step/s, range 0 to 976.3 steps/s
  POWERSTEP01_LSPD_OPT_OFF, // Low speed optimization bit, enum powerstep01_LspdOpt_t
  244.16, // Full step speed in step/s, range 7.63 to 15625 steps/s
  POWERSTEP01_BOOST_MODE_OFF, // Boost of the amplitude square wave, enum powerstep01_BoostMode_t
  281.25, // Overcurrent threshold settings via enum powerstep01_OcdTh_t
  //STEP_MODE_1_16, // Step mode settings via enum motorStepMode_t
  STEP_MODE_FULL,
  POWERSTEP01_SYNC_SEL_DISABLED, // Synch. Mode settings via enum powerstep01_SyncSel_t
  (POWERSTEP01_ALARM_EN_OVERCURRENT|
   POWERSTEP01_ALARM_EN_THERMAL_SHUTDOWN|
   POWERSTEP01_ALARM_EN_THERMAL_WARNING|
   POWERSTEP01_ALARM_EN_UVLO|
   POWERSTEP01_ALARM_EN_STALL_DETECTION|
   POWERSTEP01_ALARM_EN_SW_TURN_ON|
   POWERSTEP01_ALARM_EN_WRONG_NPERF_CMD), // Alarm settings via bitmap enum powerstep01_AlarmEn_t
  POWERSTEP01_IGATE_64mA, // Gate sink/source current via enum powerstep01_Igate_t 
  POWERSTEP01_TBOOST_0ns, // Duration of the overboost phase during gate turn-off via enum powerstep01_Tboost_t
  POWERSTEP01_TCC_500ns, // Controlled current time via enum powerstep01_Tcc_t
  POWERSTEP01_WD_EN_DISABLE, // External clock watchdog, enum powerstep01_WdEn_t  
  POWERSTEP01_TBLANK_375ns, // Duration of the blanking time via enum powerstep01_TBlank_t
  POWERSTEP01_TDT_125ns, // Duration of the dead time via enum powerstep01_Tdt_t
  /* current mode parameters */
  328.12, // Hold torque in mV, range from 7.8mV to 1000 mV
  328.12, // Running torque in mV, range from 7.8mV to 1000 mV 
  328.12, // Acceleration torque in mV, range from 7.8mV to 1000 mV
  328.12, // Deceleration torque in mV, range from 7.8mV to 1000 mV
  POWERSTEP01_TOFF_FAST_8us, //Maximum fast decay time , enum powerstep01_ToffFast_t 
  POWERSTEP01_FAST_STEP_12us, //Maximum fall step time , enum powerstep01_FastStep_t 
  3.0, // Minimum on-time in us, range 0.5us to 64us
  21.0, // Minimum off-time in us, range 0.5us to 64us 
  POWERSTEP01_CONFIG_INT_16MHZ_OSCOUT_2MHZ, // Clock setting , enum powerstep01_ConfigOscMgmt_t
  POWERSTEP01_CONFIG_SW_HARD_STOP, // External switch hard stop interrupt mode, enum powerstep01_ConfigSwMode_t
  POWERSTEP01_CONFIG_TQ_REG_TVAL_USED, // External torque regulation enabling , enum powerstep01_ConfigEnTqReg_t
  POWERSTEP01_CONFIG_VS_COMP_DISABLE, // Motor Supply Voltage Compensation enabling , enum powerstep01_ConfigEnVscomp_t 
  POWERSTEP01_CONFIG_OC_SD_DISABLE, // Over current shutwdown enabling, enum powerstep01_ConfigOcSd_t
  POWERSTEP01_CONFIG_UVLOVAL_LOW, // UVLO Threshold via powerstep01_ConfigUvLoVal_t
  POWERSTEP01_CONFIG_VCCVAL_15V, // VCC Val, enum powerstep01_ConfigVccVal_t
  POWERSTEP01_CONFIG_TSW_048us, // Switching period, enum powerstep01_ConfigTsw_t
  POWERSTEP01_CONFIG_PRED_DISABLE // Predictive current enabling , enum powerstep01_ConfigPredEn_t 
};

/* Motor Control Component. */
PowerStep01 *motor;
//DevSPI dev_spi(D11, D12, D13);

void my_flag_irq_handler(void)
{
  /* Set ISR flag. */
  motor->isrFlag = TRUE;
  /* Get the value of the status register. */
  unsigned int statusRegister = motor->get_status();
  pc.printf("    WARNING: \"FLAG\" interrupt triggered.\r\n");
  /* Check SW_F flag: if not set, the SW input is opened */
  if ((statusRegister & POWERSTEP01_STATUS_SW_F ) != 0) {
    pc.printf("    SW closed (connected to ground).\r\n");
  }
  /* Check SW_EN bit */
  if ((statusRegister & POWERSTEP01_STATUS_SW_EVN) == POWERSTEP01_STATUS_SW_EVN) {
    pc.printf("    SW turn_on event.\r\n");
  }
  /* Check Command Error flag: if set, the command received by SPI can't be */
  /* performed. This occurs for instance when a move command is sent to the */
  /* Powerstep01 while it is already running */
  if ((statusRegister & POWERSTEP01_STATUS_CMD_ERROR) == POWERSTEP01_STATUS_CMD_ERROR) {
    pc.printf("    Non-performable command detected.\r\n");
  }  
  /* Check UVLO flag: if not set, there is an undervoltage lock-out */
  if ((statusRegister & POWERSTEP01_STATUS_UVLO)==0) {
    pc.printf("    undervoltage lock-out.\r\n"); 
  }  
  /* Check thermal STATUS flags: if  set, the thermal status is not normal */
  if ((statusRegister & POWERSTEP01_STATUS_TH_STATUS)!=0) {
    //thermal status: 1: Warning, 2: Bridge shutdown, 3: Device shutdown
    pc.printf("    Thermal status: %d.\r\n", (statusRegister & POWERSTEP01_STATUS_TH_STATUS)>>11);
  }    
  /* Check OCD  flag: if not set, there is an overcurrent detection */
  if ((statusRegister & POWERSTEP01_STATUS_OCD)==0) {
    pc.printf("    Overcurrent detection.\r\n"); 
  }
  /* Reset ISR flag. */
  motor->isrFlag = FALSE;
}

void my_error_handler(uint16_t error)
{
  /* Printing to the console. */
  pc.printf("Error %d detected\r\n\n", error);
  
  /* Infinite loop */
  while (true) {
  }    
}

void init_hardware()
{
    pc.printf("Entered INIT\r\n");
    
    //initialise flipping arm
    flipping_arm_lower.calibrate();
    flipping_arm_upper.calibrate();
    
    flipping_arm_lower.position(0);
    pc.printf("Lower Pos 0\r\n");
        
    flipping_arm_upper.position(-20);
    pc.printf("Upper Pos -20\r\n");
    
    //initialise clamping arm
    //clamping_arm.calibrate();
    
    //initialise stepper motor
    //motor = new PowerStep01(D2, D4, D8, D9, D10, dev_spi);
    //if (motor->init(&init) != COMPONENT_OK) {
    //      exit(EXIT_FAILURE);
    //}

    //motor->attach_flag_irq(&my_flag_irq_handler);
    //motor->enable_flag_irq(); 
    //motor->attach_error_handler(&my_error_handler);
    
    //DevSPI dev_spi(D11, D12, D13);
    
    pc.printf("Left INIT\r\n");
}

void flip_cube()
{
    //move lower arm into position
    flipping_arm_lower.position(-15);
    pc.printf("Lower Pos -15\r\n");
    wait(0.2);
    
    //flip the cube
    flipping_arm_upper.position(45);
    pc.printf("Upper Pos 45\r\n");
        
    flipping_arm_lower.position(-40);
    pc.printf("Lower Pos -40\r\n");  
    wait(0.5);

    //return flipping arm to original position
    flipping_arm_lower.position(0);
    pc.printf("Lower Pos 0\r\n");
        
    flipping_arm_upper.position(-20);
    pc.printf("Upper Pos -20\r\n");
}

////////////// Clamp functions //////////////////
void lower_clamp()
{
    Servo clamping_arm(PA_7);
    clamping_arm.calibrate();
    clamping_arm.position(15);
}

void raise_clamp()
{
    Servo clamping_arm(PA_7);
    clamping_arm.calibrate();
    clamping_arm.position(60);
}
/////////////////////////////////////////////////

void rotate_motor_90_degrees_clockwise()
{
    motor->move(StepperMotor::FWD, 51);
    motor->wait_while_active();
}

////////////// Stepper functions //////////////////
void rotate_cube_90_degrees_clockwise()
{
    //initialise stepper motor
    pc.printf("Entered rotate_cube_90_degrees_clockwise\r\n");
    DevSPI dev_spi(D11, D12, D13);
    
    pc.printf("INIT SPI\r\n");
    
    motor = new PowerStep01(D2, D4, D8, D9, D10, dev_spi);
    pc.printf("Create new motor\r\n");
    
    motor->reset_command();
    pc.printf("Reset command\r\n");
    
    if (motor->init(&init) != COMPONENT_OK) {
        pc.printf("Init motor failed\r\n");
        exit(EXIT_FAILURE);
    }
    pc.printf("Init motor\r\n");
    
    motor->attach_flag_irq(&my_flag_irq_handler);
    pc.printf("attach_flag_irq\r\n");
    motor->enable_flag_irq(); 
    pc.printf("enable_flag_irq\r\n");
    motor->attach_error_handler(&my_error_handler);
    pc.printf("attach_error_handler\r\n");
    
    
    motor->move(StepperMotor::FWD, 50);
    pc.printf("move\r\n");
    motor->wait_while_active();
    pc.printf("wait_while_active\r\n");
    
    
    delete motor;
}

void rotate_cube_180_degrees()
{
    motor->move(StepperMotor::FWD, 100);
    motor->wait_while_active();
}

void rotate_cube_90_degrees_anticlockwise()
{
    motor->move(StepperMotor::BWD, 50);
    motor->wait_while_active();
}
////////////////////////////////////////////////////

void twist_cube_90_degrees_clockwise()
{
    pc.printf("Entered twist_cube_90_degrees_clockwise\r\n");
    
    //lower_clamp();
    wait_ms(500);
    rotate_cube_90_degrees_clockwise();
    wait_ms(2000);
    
    //raise_clamp();
    
    pc.printf("Left twist_cube_90_degrees_clockwise\r\n");
    
}
